/*
******************************************************************************************************************************************************
**
**  Les capteurs de température DS18B20
**   
**   - Découverte des capteurs
**   - Lecture des températures des capteurs présents
**
**
******************************************************************************************************************************************************
*/



/* Déclarer les bibliothèques qui sont utilisées
*/
#include <OneWire.h>                  // Librairie pour capteur OneWire

/* Déclaration des constantes, variables et objets
*/
const int pinOneWire = 2;             // Broche utilisée pour les capteurs
const int maxSensors = 5;             // Nombre maximal de capteurs possible dans ce programme
const byte onewireModeRead = 0xBE;    // Code pour initier la lecture des capteurs OneWire
const byte onewireModeMeasure = 0x44; // Code pour d�clencher la lecture des capteurs OneWire

byte data[12];                        // Tableau de 12 octets pour lecture des 9 registres de RAM et des 3 registres d'EEPROM du capteur One Wire
byte adresse[8];                      // Tableau de 8 octets pour stockage du code d'adresse 64 bits du composant One Wire
int compt;                            // Variable de comptage du nombre de capteurs sur le bus One Wire
byte sensors[maxSensors][8];          // Tableau de 5 addresses. Augmenter si plus de 5 capteurs

OneWire  oneWire(pinOneWire);         // crée un objet One Wire sur la broche voulue



/******************************************************************************************************************************
******************************************************************************************************************************* 
**
**    Fonction obligatoire "setup"
**
*/
void setup() {
  Serial.begin(115200);                               // initialise connexion série à 115200 bauds

  // Détection des capteurs présents sur le bus One Wire
  Serial.println("*** Liste des éléments presents sur le bus 1-wire *** ");
  Serial.println(); 
  compt = 0;
  // Tant qu'un nouveau capteur est détecté
  while (oneWire.search(adresse)== true) {                    
    // la fonction search renvoie la valeur VRAI si un élément 1-wire est trouvé. Stocke son adresse dans le tableau adresse
    // adresse correspond à l'adresse de début du tableau adresse[8] déclaré ... 
    for(int i = 0; i < 8; i++) {
      sensors[compt][i] = adresse[i];
    }
    compt += 1;                                       // Incrémente la variable de comptage du nombre de compteurs
    Serial.print ("Capteur numero ");
    Serial.println (compt);
    printOneWire(adresse);                            // Affichage de chaque capteur
    Serial.println("------------"); 
    Serial.println(); 
  }
  // Message récapitulatif
  if (compt==0) {
    // si aucun capteur n'a été détecté
    Serial.println("Aucun capteur present");
  }
  else { // si au moins 1 capteur a été détecté
    Serial.print(compt);                              // Affiche nombre de capteurs détectés
    Serial.print(" capteur(s) detecte(s)"); 
  }
  Serial.print(" sur le bus 1-wire de la broche n° "); 
  Serial.println(pinOneWire);
  Serial.println("");
  Serial.println("");
}



/******************************************************************************************************************************
******************************************************************************************************************************* 
**
**    Fonction obligatoire "loop"
**
*/
void loop() {
  // Boucle sur chacun des capteurs
  for(int i = 0; i < compt; i++) {
    Serial.print("Capteur n° ");
    Serial.print(i+1);
    Serial.print(" : ");
    Serial.print(onewireMesureTemp(sensors[i]));
    Serial.print(" °C  (");
    printOneWireAdress(sensors[i]);  
    Serial.println(")");
    delay(50);
  }
  Serial.println("");
  //delay(2000);
}


/*************************************************************
**   Lecture d'une température sur le capteur dont l'adresse est en param�tre
*/
float onewireMesureTemp(byte addr[8]) { 
  byte data[12];        // Tableau de 12 octets pour lecture des 9 registres de RAM et des 3 registres d'EEPROM du capteur One Wire
  int tempet = 0;       // Variable pour résultat brute de la mesure
  float tempetf = 0.0;  // Variable pour résultat à virgule de la mesure
  oneWire.reset();
  oneWire.select(addr);
  oneWire.write(onewireModeMeasure, 1);   // start conversion, with parasite power on at the end
  delay(800);
  oneWire.reset();
  oneWire.select(addr);    
  oneWire.write(onewireModeRead);         // Récupération des valeurs de la RAM du capteur

  // Lire les 9 octets de la RAM (appel� Scratchpad) 
  for ( int i = 0; i < 9; i++) {          // 9 octets de RAM stock�s dans 9 octets
    data[i] = oneWire.read();             // lecture de l'octet de rang i stocké dans tableau data
   }
  tempetf = ((data[1] << 8) | data[0]) * 0.0625;
  return (tempetf);
}



/*************************************************************
*   printOneWire
*/
void printOneWire(byte adrs[8]) {
  // -- Test du type de capteur -- 
  // Le type du capteur est donné par le 1er octet du code adresse 64 bits
  // Valeur 0x28 pour capteur type DS18B20, 0x10 pour type DS18S20, 0x22 pour type DS1820
  Serial.print ("  Type :     ");
  if (adrs[0]==0x28) {
    Serial.println ("Capteur temperature DS18B20.");
  }
  Serial.print ("  Adresse :  ");
  printOneWireAdress(adrs);
  // -- Adresse --
  Serial.println("");
  //-- Code CRC ---
  // le dernier octet de l'adresse 64bits est un code de contrôle CRC 
  // à l'aide de la fonction crc8 on peut vérifier si ce code est valide
  // vérification validité code CRC de l'adresse 64 bits
  Serial.print ("  Code CRC : "); 
  if (oneWire.crc8( adrs, 7) == adrs[7]) {      // le code CRC de l'adresse 64 bits est le 8ème octet de l'adresse (index 7 du tableau)
    Serial.println ("VALIDE !"); 
  }
  else {
    Serial.println ("NON VALIDE !");     
  }
}

/*************************************************************
*   printOneWire
*/
void printOneWireAdress(byte adrs[8]) {
  for(int i = 0; i < 8; i++) {     // L'adresse renvoyée par la fonction search est stockée sur 8 octets, donc 8 boucles
    if (adrs[i] < 16) {
      Serial.print('0');           // Pour affichage des O poids fort au format hexadécimal
    }
    Serial.print(adrs[i], HEX);    // Affiche 1 à 1 les 8 octets du tableau adresse au format hexadécimal
    Serial.print(" ");
  }
}


